package Shared;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;
import java.util.zip.Deflater;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;


public class Zipper {
	
	private static final int BUFFER_SIZE = 2048;

	public void uncompress(ZipFile zipFile, String target) throws IOException{
	
		FileInputStream fIn = new FileInputStream(zipFile.getName());
		ZipInputStream zipIn = new ZipInputStream(new BufferedInputStream(fIn));
		ZipEntry entry;
		byte[] buffer = new byte[BUFFER_SIZE];
		
		while((entry = zipIn.getNextEntry()) != null){
			//see if the directory exists. Otherwise, create a new directory
			int index = entry.getName().lastIndexOf(File.separatorChar);
			String directory = entry.getName().substring(0, index);
			File tempDir = new File(target+File.separator+directory);
			tempDir.mkdirs();
			
			FileOutputStream fOut = new  FileOutputStream(target+File.separator+entry.getName());
			BufferedOutputStream bOut = new BufferedOutputStream(fOut,BUFFER_SIZE);	
		
			int len;
			while((len = zipIn.read(buffer,0,BUFFER_SIZE)) >=0)
				bOut.write(buffer,0,len);
			
			bOut.flush();
			bOut.close();
		}
		zipIn.close();
	}
	
	public void compress(File directory, String target) throws IOException{	
		Set<String> files = getFilesRecursively(directory);
		
		FileOutputStream fOut = new FileOutputStream(target);
		ZipOutputStream zipOut = new ZipOutputStream(new BufferedOutputStream(fOut));
		zipOut.setLevel(Deflater.BEST_COMPRESSION);
		byte[] buffer = new byte[BUFFER_SIZE];
		
		for(String file : files){	
			FileInputStream fIn = new FileInputStream(file);
			BufferedInputStream bIn = new BufferedInputStream(fIn,BUFFER_SIZE);
			ZipEntry entry = new ZipEntry(file);
			zipOut.putNextEntry(entry);
			
			int len;
			while((len = bIn.read(buffer, 0, BUFFER_SIZE)) != -1)
				zipOut.write(buffer,0,len);
			
			
			bIn.close();	
		}
		zipOut.close();
	}
	
	private Set<String> getFilesRecursively(File directory){
		Set<String> files = new HashSet<String>();
		recurse(directory, files);
		return files;
	}
	
	private void recurse(File dir, Set<String> files){
		if(dir.isFile()){
			files.add(dir.getPath());
			return;
		}
		
		for(File f : dir.listFiles())
			recurse(f, files);
	}
}
